#!/usr/bin/env node

/**
 * ZAI MCP Server - AI-to-AI Loop System with Multi-Provider Support
 * Supports OpenRouter, Anthropic, and DeepSeek APIs with automatic failover
 */

import { Server } from '@modelcontextprotocol/sdk/server/index.js';
import { StdioServerTransport } from '@modelcontextprotocol/sdk/server/stdio.js';
import { CallToolRequestSchema, ListToolsRequestSchema } from '@modelcontextprotocol/sdk/types.js';
import { MultiProviderAI } from './multiProviderAI.js';
import { ZAIDataCollector } from './data-collection-system.js';
import { AIVotingManager } from './aiVotingManager.js';
import { TaskManager } from './taskManager.js';
import { DeepThinking } from './deepThinking.js';
import { ParallelImplementation } from './parallelImplementation.js';
import { AIAgentCoordinator } from './aiAgentCoordinator.js';

class ZAIMCPServer {
    constructor() {
        this.server = new Server(
            {
                name: 'zai-mcp-server',
                version: '2.0.0',
            },
            {
                capabilities: {
                    tools: {},
                },
            }
        );

        // Initialize AI providers, voting system, and data collection
        try {
            this.multiAI = new MultiProviderAI();
            this.votingManager = new AIVotingManager(this.multiAI);
            this.dataCollector = new ZAIDataCollector();

            // Initialize advanced task management components
            this.taskManager = new TaskManager(this.multiAI, this.votingManager);
            this.deepThinking = new DeepThinking(this.multiAI, this.votingManager);
            this.parallelImplementation = new ParallelImplementation(this.multiAI, this.taskManager, this.deepThinking);

            // Initialize AI Agent Coordinator for enhanced interaction management
            this.aiCoordinator = new AIAgentCoordinator(this);
        } catch (error) {
            console.error('❌ Failed to initialize components:', error);
            throw error;
        }
        
        // Active loops storage
        this.activeLoops = new Map();
        this.loopCounter = 0;
        
        // Agent state management
        this.agentBusy = false;
        this.pendingResponses = [];

        this.setupToolHandlers();
        this.setupErrorHandling();

        // 🆓 FREE VERSION - No license validation required
        console.error('🆓 ZAI MCP Server - FREE VERSION');
        console.error('📊 Data collection enabled for AI training');
        console.error('💡 Help us improve AI by using this free service!');
    }

    setupToolHandlers() {
        // List available tools
        this.server.setRequestHandler(ListToolsRequestSchema, async () => ({
            tools: [
                {
                    name: 'activate_infinite_loop',
                    description: 'Start AI-to-AI improvement loops for continuous development',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Activation message with topic (format: "actloop [topic]")'
                            },
                            aiToAi: {
                                type: 'boolean',
                                description: 'Enable AI-to-AI communication mode',
                                default: true
                            },
                            interval: {
                                type: 'number',
                                description: 'Loop interval in milliseconds',
                                default: 5000
                            },
                            maxIterations: {
                                type: 'number',
                                description: 'Maximum iterations before auto-stop',
                                default: 999999
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'stop_ai_loops',
                    description: 'Stop all active AI-to-AI loops',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Stop command (e.g., "stploop")'
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'list_active_loops',
                    description: 'View running loops',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_provider_status',
                    description: 'Check provider status',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'reset_ai_providers',
                    description: 'Reset failed providers',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_prompts',
                    description: 'Get AI-generated prompts',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Maximum number of prompts to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'acknowledge_agent_response',
                    description: 'Process AI responses',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            loopId: {
                                type: 'string',
                                description: 'The ID of the loop to acknowledge'
                            },
                            agentResponse: {
                                type: 'string',
                                description: 'The agent response text'
                            }
                        },
                        required: ['loopId']
                    }
                },
                {
                    name: 'ai_voting_request',
                    description: 'Submit prompt for multi-model AI consensus voting',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            prompt: {
                                type: 'string',
                                description: 'The prompt to submit for AI voting'
                            },
                            panel: {
                                type: 'string',
                                description: 'Voting panel to use (general, coding, reasoning, premium)',
                                default: 'general'
                            },
                            strategy: {
                                type: 'string',
                                description: 'Voting strategy (majority, consensus, weighted, unanimous)',
                                default: 'consensus'
                            },
                            maxAgents: {
                                type: 'number',
                                description: 'Maximum number of agents to use',
                                default: 5
                            }
                        },
                        required: ['prompt']
                    }
                },
                {
                    name: 'get_voting_history',
                    description: 'View recent voting sessions',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Number of recent sessions to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_agent_performance',
                    description: 'Check AI agent performance statistics',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            agentId: {
                                type: 'string',
                                description: 'Specific agent ID to check (optional)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'analyze_task_breakdown',
                    description: 'Analyze a topic and break it down into actionable subtasks with prioritization',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            topic: {
                                type: 'string',
                                description: 'The topic or project to analyze and break down'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for the analysis (optional)'
                            }
                        },
                        required: ['topic']
                    }
                },
                {
                    name: 'deep_think_implementation',
                    description: 'Perform deep thinking analysis on a specific task to explore solutions and create implementation plans',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            taskId: {
                                type: 'string',
                                description: 'ID of the task to analyze'
                            },
                            taskName: {
                                type: 'string',
                                description: 'Name of the task (if taskId not available)'
                            },
                            taskDescription: {
                                type: 'string',
                                description: 'Description of the task (if taskId not available)'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for deep thinking (optional)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'parallel_execute_tasks',
                    description: 'Execute multiple tasks in parallel with intelligent coordination and dependency management',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            breakdownId: {
                                type: 'string',
                                description: 'ID of the task breakdown to execute'
                            },
                            executionStrategy: {
                                type: 'string',
                                description: 'Execution strategy (parallel, sequential, hybrid)',
                                default: 'parallel'
                            },
                            maxConcurrency: {
                                type: 'number',
                                description: 'Maximum number of concurrent tasks',
                                default: 5
                            }
                        },
                        required: ['breakdownId']
                    }
                }
            ]
        }));

        // Handle tool calls
        this.server.setRequestHandler(CallToolRequestSchema, async (request) => {
            const { name, arguments: args } = request.params;

            try {
                switch (name) {
                    case 'activate_infinite_loop':
                        return await this.handleActivateLoop(args);

                    case 'stop_ai_loops':
                        return await this.handleStopAILoops(args);

                    case 'list_active_loops':
                        return await this.handleListActiveLoops(args);

                    case 'get_ai_provider_status':
                        return await this.handleGetAIProviderStatus(args);

                    case 'reset_ai_providers':
                        return await this.handleResetAIProviders(args);

                    case 'get_ai_prompts':
                        return await this.handleGetAIPrompts(args);

                    case 'acknowledge_agent_response':
                        return await this.handleAcknowledgeAgentResponse(args);

                    case 'ai_voting_request':
                        return await this.handleAIVotingRequest(args);

                    case 'get_voting_history':
                        return await this.handleGetVotingHistory(args);

                    case 'get_agent_performance':
                        return await this.handleGetAgentPerformance(args);

                    case 'analyze_task_breakdown':
                        return await this.handleAnalyzeTaskBreakdown(args);

                    case 'deep_think_implementation':
                        return await this.handleDeepThinkImplementation(args);

                    case 'parallel_execute_tasks':
                        return await this.handleParallelExecuteTasks(args);

                    default:
                        throw new Error(`Unknown tool: ${name}`);
                }
            } catch (error) {
                return {
                    content: [
                        {
                            type: 'text',
                            text: `Error executing tool ${name}: ${error.message}`
                        }
                    ],
                    isError: true
                };
            }
        });
    }

    async handleActivateLoop(args) {
        const { message, aiToAi = true, interval = 5000, maxIterations = 999999 } = args;

        if (!message || typeof message !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid message parameter. Please provide a string message with format: "actloop [topic]"'
                }]
            };
        }

        // Check if message matches activation pattern
        if (!message.toLowerCase().startsWith('actloop ')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid activation message format. Please use format: "actloop [topic]" (e.g., "actloop improve my React component")'
                }]
            };
        }

        const topic = message.substring(8).trim();
        if (!topic) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ No topic specified. Please provide a topic after "actloop " (e.g., "actloop improve my React component")'
                }]
            };
        }

        try {
            const loopId = `loop-${++this.loopCounter}`;
            const loop = {
                id: loopId,
                topic,
                aiToAi,
                interval,
                maxIterations,
                currentIteration: 0,
                status: 'running',
                startTime: new Date().toISOString(),
                lastActivity: new Date().toISOString()
            };

            this.activeLoops.set(loopId, loop);

            // Start the enhanced AI-to-AI loop with task management
            this.startEnhancedAILoop(loop);

            return {
                content: [{
                    type: 'text',
                    text: `✅ AI-to-AI loop started successfully!\n\n🔄 Loop ID: ${loopId}\n📝 Topic: ${topic}\n🤖 AI-to-AI Mode: ${aiToAi ? 'Enabled' : 'Disabled'}\n⏱️ Interval: ${interval}ms\n🔢 Max Iterations: ${maxIterations}\n\n🚀 The loop is now running and will continuously improve the topic using AI-to-AI collaboration.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to start loop: ${error.message}`
                }]
            };
        }
    }

    async startAILoop(loop) {
        const runIteration = async () => {
            if (loop.status !== 'running' || loop.currentIteration >= loop.maxIterations) {
                this.activeLoops.delete(loop.id);
                return;
            }

            try {
                loop.currentIteration++;
                loop.lastActivity = new Date().toISOString();

                // Generate AI improvement using multi-provider system
                const prompt = `Improve the following topic: "${loop.topic}".

Current iteration: ${loop.currentIteration}
Provide a specific, actionable improvement suggestion.`;

                const response = await this.multiAI.makeRequest(prompt, {
                    maxTokens: 500,
                    temperature: 0.7
                });

                // Collect data for training
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt,
                    response: response.content,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration
                    },
                    success: true,
                    responseTime: response.responseTime,
                    loopIteration: loop.currentIteration,
                    model: response.model,
                    provider: response.provider,
                    sessionId: loop.id
                });

                console.error(`[AI-TO-AI] Loop ${loop.id} - Iteration ${loop.currentIteration}: ${response.content.substring(0, 100)}...`);

                // Schedule next iteration
                setTimeout(runIteration, loop.interval);

            } catch (error) {
                console.error(`[AI-TO-AI] Error in loop ${loop.id}:`, error);

                // Collect error data
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt: `Improve: ${loop.topic}`,
                    response: null,
                    context: { loopId: loop.id, topic: loop.topic, iteration: loop.currentIteration },
                    success: false,
                    errors: [error.message],
                    sessionId: loop.id
                });

                // Retry after longer interval
                setTimeout(runIteration, loop.interval * 2);
            }
        };

        // Start the first iteration
        setTimeout(runIteration, 1000);
    }

    async startEnhancedAILoop(loop) {
        console.log(`🚀 Starting enhanced AI-to-AI loop with task management for: ${loop.topic}`);

        let currentBreakdown = null;
        let currentExecution = null;

        const runEnhancedIteration = async () => {
            if (loop.status !== 'running' || loop.currentIteration >= loop.maxIterations) {
                this.activeLoops.delete(loop.id);
                return;
            }

            try {
                loop.currentIteration++;
                loop.lastActivity = new Date().toISOString();

                // Phase 1: Task breakdown (every 5 iterations or first iteration)
                if (loop.currentIteration === 1 || loop.currentIteration % 5 === 0) {
                    console.log(`🎯 [Loop ${loop.id}] Performing task breakdown analysis`);

                    currentBreakdown = await this.taskManager.analyzeTaskBreakdown(loop.topic, {
                        loopId: loop.id,
                        iteration: loop.currentIteration,
                        previousBreakdown: currentBreakdown?.id
                    });

                    console.log(`📋 [Loop ${loop.id}] Identified ${currentBreakdown.subtasks.length} subtasks`);
                }

                // Phase 2: Deep thinking on priority tasks (every 3 iterations)
                if (currentBreakdown && (loop.currentIteration % 3 === 0)) {
                    const prioritizedTasks = this.taskManager.prioritizeTasks(currentBreakdown);
                    const topTask = prioritizedTasks[0];

                    if (topTask) {
                        console.log(`🧠 [Loop ${loop.id}] Deep thinking on: ${topTask.name}`);

                        const thinkingSession = await this.deepThinking.deepThinkImplementation(topTask, {
                            loopId: loop.id,
                            iteration: loop.currentIteration
                        });

                        console.log(`💡 [Loop ${loop.id}] Deep thinking completed in ${thinkingSession.duration}ms`);
                    }
                }

                // Phase 3: Parallel execution (every 7 iterations)
                if (currentBreakdown && (loop.currentIteration % 7 === 0)) {
                    console.log(`⚡ [Loop ${loop.id}] Starting parallel execution`);

                    const executionPlan = this.taskManager.getExecutionPlan(currentBreakdown);
                    currentExecution = await this.parallelImplementation.parallelExecuteTasks(currentBreakdown, executionPlan);

                    console.log(`✅ [Loop ${loop.id}] Parallel execution completed: ${currentExecution.completedTasks.size} tasks done`);

                    // Add discovered tasks to the breakdown
                    if (currentExecution.discoveredTasks.length > 0) {
                        currentBreakdown.subtasks.push(...currentExecution.discoveredTasks);
                        console.log(`💡 [Loop ${loop.id}] Added ${currentExecution.discoveredTasks.length} discovered tasks`);
                    }
                }

                // Phase 4: Regular AI improvement (every iteration)
                const prompt = `Continue improving: "${loop.topic}".

Current iteration: ${loop.currentIteration}
${currentBreakdown ? `Active breakdown: ${currentBreakdown.subtasks.length} subtasks identified` : ''}
${currentExecution ? `Last execution: ${currentExecution.completedTasks.size} tasks completed` : ''}

Provide the next improvement or insight.`;

                const response = await this.multiAI.makeRequest(prompt, {
                    maxTokens: 500,
                    temperature: 0.7
                });

                // Collect comprehensive data
                await this.dataCollector.collectInteraction({
                    type: 'enhanced-ai-loop',
                    prompt,
                    response: response.content,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration,
                        hasBreakdown: !!currentBreakdown,
                        hasExecution: !!currentExecution,
                        subtaskCount: currentBreakdown?.subtasks.length || 0,
                        completedTasks: currentExecution?.completedTasks.size || 0
                    },
                    success: true,
                    responseTime: response.responseTime,
                    loopIteration: loop.currentIteration,
                    model: response.model,
                    provider: response.provider,
                    sessionId: loop.id
                });

                console.error(`[ENHANCED-AI-LOOP] ${loop.id} - Iteration ${loop.currentIteration}: ${response.content.substring(0, 100)}...`);

                // Schedule next iteration
                setTimeout(runEnhancedIteration, loop.interval);

            } catch (error) {
                console.error(`[ENHANCED-AI-LOOP] Error in loop ${loop.id}:`, error);

                // Collect error data
                await this.dataCollector.collectInteraction({
                    type: 'enhanced-ai-loop',
                    prompt: `Enhanced improve: ${loop.topic}`,
                    response: null,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration,
                        error: error.message
                    },
                    success: false,
                    errors: [error.message],
                    sessionId: loop.id
                });

                // Retry after longer interval
                setTimeout(runEnhancedIteration, loop.interval * 2);
            }
        };

        // Start the first enhanced iteration
        setTimeout(runEnhancedIteration, 2000);
    }

    async handleStopAILoops(args) {
        const { message } = args;

        if (!message || !message.toLowerCase().includes('stploop')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid stop command. Please use "stploop" to stop AI-to-AI loops.'
                }]
            };
        }

        const stoppedCount = this.activeLoops.size;

        // Stop all active loops
        for (const [, loop] of this.activeLoops) {
            loop.status = 'stopped';
        }
        this.activeLoops.clear();

        // Flush any pending data
        await this.dataCollector.flushData();

        return {
            content: [{
                type: 'text',
                text: `✅ All AI-to-AI loops stopped successfully!\n\n🛑 Stopped ${stoppedCount} active loop(s)\n📊 Data collection completed for all loops`
            }]
        };
    }

    async handleListActiveLoops() {
        if (this.activeLoops.size === 0) {
            return {
                content: [{
                    type: 'text',
                    text: '📋 No active loops currently running.'
                }]
            };
        }

        const loopsList = Array.from(this.activeLoops.values()).map(loop =>
            `🔄 **${loop.id}**\n` +
            `   📝 Topic: ${loop.topic}\n` +
            `   🔢 Iteration: ${loop.currentIteration}\n` +
            `   ⏱️ Started: ${new Date(loop.startTime).toLocaleString()}\n` +
            `   📊 Status: ${loop.status === 'running' ? '🟢 Active' : '🔴 Stopped'}`
        ).join('\n\n');

        return {
            content: [{
                type: 'text',
                text: `📋 **Active AI-to-AI Loops (${this.activeLoops.size})**\n\n${loopsList}`
            }]
        };
    }

    async handleGetAIProviderStatus() {
        try {
            const status = this.multiAI.getStatus();

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Provider Status**\n\n` +
                          `🎯 **Current Provider:** ${status.currentProvider || 'None'}\n` +
                          `📊 **Success Rate:** ${status.successCount}/${status.totalRequests} (${status.successRate}%)\n` +
                          `⚡ **Available Providers:**\n` +
                          Object.entries(status.providers).map(([, provider]) =>
                              `   ${provider.enabled ? '✅' : '❌'} ${provider.name} (${provider.apiKeys} key(s))`
                          ).join('\n') + '\n\n' +
                          `🔄 **Request Statistics:**\n` +
                          `   Total Requests: ${status.totalRequests}\n` +
                          `   Successful: ${status.successCount}\n` +
                          `   Failed: ${status.errorCount}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get provider status: ${error.message}`
                }]
            };
        }
    }

    async handleResetAIProviders() {
        try {
            this.multiAI.resetProviders();

            return {
                content: [{
                    type: 'text',
                    text: `✅ AI providers reset successfully!\n\n🔄 All failed providers have been reset and are available for retry.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to reset providers: ${error.message}`
                }]
            };
        }
    }

    async handleGetAIPrompts(args) {
        const { limit = 5 } = args;

        try {
            // Get contextual prompts from AI Agent Coordinator
            const contextualPrompts = this.aiCoordinator.getContextualPrompts(limit);

            // If no contextual prompts available, generate general improvement prompts
            if (contextualPrompts.length === 0) {
                const generalPrompts = await this.generateGeneralImprovementPrompts(limit);
                contextualPrompts.push(...generalPrompts);
            }

            // Ensure we have at least some prompts
            const prompts = contextualPrompts.slice(0, limit);

            if (prompts.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: '🤖 **AI-Generated Prompts**\n\n' +
                              '💡 No active operations detected. Start a task analysis or execution to receive contextual prompts.\n\n' +
                              '**Available Operations:**\n' +
                              '• `analyze_task_breakdown` - Get task-specific improvement suggestions\n' +
                              '• `deep_think_implementation` - Receive implementation optimization prompts\n' +
                              '• `parallel_execute_tasks` - Get execution monitoring and optimization prompts'
                    }]
                };
            }

            const promptText = prompts.map((prompt, index) => {
                const priorityIcon = prompt.priority === 'high' ? '🔥' : prompt.priority === 'medium' ? '⚡' : '💡';
                const typeIcon = this.getPromptTypeIcon(prompt.type);

                return `${index + 1}. ${priorityIcon} **${prompt.type.replace('_', ' ').toUpperCase()}**\n` +
                       `   ${typeIcon} ${prompt.content}\n` +
                       `   📋 Context: ${prompt.context}\n` +
                       `   ⏰ Generated: ${new Date(prompt.timestamp).toLocaleString()}\n` +
                       `   🎯 Priority: ${prompt.priority.toUpperCase()}`;
            }).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI-Generated Contextual Prompts (${prompts.length})**\n\n` +
                          `🔄 Last updated: ${new Date(this.aiCoordinator.lastPromptCheck).toLocaleString()}\n` +
                          `⚡ Active operations: ${this.aiCoordinator.getAllActiveOperations().length}\n\n` +
                          promptText + '\n\n' +
                          `💡 **Tip:** These prompts are generated based on your current AI agent operations and update every 30 seconds.`
                }]
            };
        } catch (error) {
            console.error('❌ Failed to get AI prompts:', error);
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to generate AI prompts: ${error.message}\n\nPlease try again or start an AI agent operation to receive contextual prompts.`
                }]
            };
        }
    }

    async generateGeneralImprovementPrompts(limit = 5) {
        // Generate general improvement prompts when no contextual prompts are available
        const currentTime = Date.now();
        const prompts = [];

        const generalSuggestions = [
            {
                type: 'performance_optimization',
                content: 'Consider implementing caching strategies to improve response times for frequently accessed data',
                priority: 'medium'
            },
            {
                type: 'code_quality',
                content: 'Review current implementation for opportunities to refactor complex functions into smaller, more maintainable units',
                priority: 'low'
            },
            {
                type: 'error_handling',
                content: 'Enhance error handling mechanisms to provide better user feedback and system resilience',
                priority: 'high'
            },
            {
                type: 'monitoring',
                content: 'Add comprehensive logging and monitoring to track system performance and identify potential issues',
                priority: 'medium'
            },
            {
                type: 'security',
                content: 'Conduct security review to ensure proper input validation and protection against common vulnerabilities',
                priority: 'high'
            },
            {
                type: 'documentation',
                content: 'Update documentation to reflect recent changes and improve developer onboarding experience',
                priority: 'low'
            },
            {
                type: 'testing',
                content: 'Expand test coverage to include edge cases and integration scenarios for better reliability',
                priority: 'medium'
            }
        ];

        // Select random suggestions up to the limit
        const selectedSuggestions = generalSuggestions
            .sort(() => Math.random() - 0.5)
            .slice(0, limit);

        selectedSuggestions.forEach((suggestion, index) => {
            prompts.push({
                id: `prompt-general-${currentTime}-${index}`,
                content: suggestion.content,
                type: suggestion.type,
                context: 'general',
                priority: suggestion.priority,
                timestamp: new Date().toISOString()
            });
        });

        return prompts;
    }

    getPromptTypeIcon(type) {
        const icons = {
            'optimization': '⚡',
            'analysis': '🔍',
            'risk_management': '⚠️',
            'exploration': '🚀',
            'edge_case_analysis': '🎯',
            'scalability': '📈',
            'performance': '🏃',
            'synchronization': '🔄',
            'load_balancing': '⚖️',
            'general_improvement': '💡',
            'user_experience': '👤',
            'performance_optimization': '⚡',
            'code_quality': '🔧',
            'error_handling': '🛡️',
            'monitoring': '📊',
            'security': '🔒',
            'documentation': '📚',
            'testing': '🧪'
        };

        return icons[type] || '💡';
    }

    async handleAcknowledgeAgentResponse(args) {
        const { loopId, agentResponse } = args;

        if (!loopId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing loopId parameter.'
                }]
            };
        }

        const loop = this.activeLoops.get(loopId);
        if (!loop) {
            // For testing purposes, allow acknowledgment even if loop doesn't exist
            console.log(`⚠️ Loop ${loopId} not found, but acknowledging anyway for testing`);
        }

        // Record the agent response for data collection
        if (agentResponse) {
            await this.dataCollector.collectInteraction({
                type: 'agent-response',
                prompt: `Loop ${loopId} acknowledgment`,
                response: agentResponse,
                context: { loopId, topic: loop?.topic || 'unknown' },
                success: true,
                sessionId: loopId
            });
        }

        return {
            content: [{
                type: 'text',
                text: `✅ Agent response acknowledged for loop ${loopId}.\n\n🔄 Loop will continue with next iteration.`
            }]
        };
    }

    async handleAIVotingRequest(args) {
        const { prompt, panel = 'general', strategy = 'consensus', maxAgents = 5 } = args;

        if (!prompt || typeof prompt !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid prompt parameter. Please provide a string prompt for AI voting.'
                }]
            };
        }

        try {
            console.log(`🗳️ Starting AI voting session: Panel=${panel}, Strategy=${strategy}`);

            const votingResult = await this.votingManager.conductVoting(prompt, {
                panel,
                strategy,
                maxAgents,
                enableDebug: true
            });

            // Collect voting data for training
            await this.dataCollector.collectInteraction({
                type: 'ai-voting',
                prompt,
                response: votingResult.winningResponse.content,
                context: {
                    panel,
                    strategy,
                    totalAgents: votingResult.totalAgents,
                    consensusScore: votingResult.consensusScore
                },
                success: true,
                votingData: {
                    votes: votingResult.votes,
                    responses: votingResult.responses.length,
                    winner: votingResult.winningResponse.agentId
                },
                sessionId: `voting-${Date.now()}`
            });

            return {
                content: [{
                    type: 'text',
                    text: `🗳️ **AI Voting Results**\n\n` +
                          `📝 **Prompt:** ${prompt}\n\n` +
                          `🏆 **Winning Response** (${votingResult.winningResponse.agentId}):\n` +
                          `${votingResult.winningResponse.content}\n\n` +
                          `📊 **Voting Details:**\n` +
                          `   Panel: ${panel}\n` +
                          `   Strategy: ${strategy}\n` +
                          `   Total Agents: ${votingResult.totalAgents}\n` +
                          `   Consensus Score: ${(votingResult.consensusScore * 100).toFixed(1)}%\n` +
                          `   Duration: ${votingResult.duration}ms\n\n` +
                          `🗳️ **Vote Distribution:**\n` +
                          votingResult.votes.map(vote =>
                              `   ${vote.voter.id}: Response ${vote.selectedResponseIndex + 1} (confidence: ${vote.confidence}/10)`
                          ).join('\n')
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ AI voting failed: ${error.message}\n\nPlease check your API keys and try again.`
                }]
            };
        }
    }

    async handleGetVotingHistory(args) {
        const { limit = 5 } = args;

        try {
            const history = this.votingManager.getVotingHistory(limit);

            if (history.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: '📋 No voting sessions found. Start a voting session with the "ai_voting_request" tool.'
                    }]
                };
            }

            const historyText = history.map((session, index) =>
                `${index + 1}. **Session ${session.sessionId}**\n` +
                `   📝 Prompt: ${session.prompt.substring(0, 100)}${session.prompt.length > 100 ? '...' : ''}\n` +
                `   🏆 Winner: ${session.winner}\n` +
                `   📊 Panel: ${session.panel} | Strategy: ${session.strategy}\n` +
                `   ⏱️ Time: ${new Date(session.timestamp).toLocaleString()}\n` +
                `   🎯 Consensus: ${(session.consensusScore * 100).toFixed(1)}%`
            ).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🗳️ **Recent Voting Sessions (${history.length})**\n\n${historyText}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get voting history: ${error.message}`
                }]
            };
        }
    }

    async handleGetAgentPerformance(args) {
        const { agentId } = args;

        try {
            const performance = this.votingManager.getAgentPerformance(agentId);

            if (agentId && !performance[agentId]) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Agent "${agentId}" not found or has no performance data.`
                    }]
                };
            }

            let performanceText;
            if (agentId) {
                const agent = performance[agentId];
                performanceText = `🤖 **Agent Performance: ${agentId}**\n\n` +
                    `📊 **Statistics:**\n` +
                    `   Total Responses: ${agent.totalResponses}\n` +
                    `   Wins: ${agent.wins}\n` +
                    `   Win Rate: ${(agent.winRate * 100).toFixed(1)}%\n` +
                    `   Average Confidence: ${agent.averageConfidence.toFixed(1)}/10\n` +
                    `   Average Response Time: ${agent.averageResponseTime}ms\n\n` +
                    `🏆 **Recent Performance:**\n` +
                    `   Last 5 Sessions: ${agent.recentWins}/5 wins\n` +
                    `   Trend: ${agent.trend}`;
            } else {
                const agents = Object.entries(performance);
                if (agents.length === 0) {
                    return {
                        content: [{
                            type: 'text',
                            text: '📊 No agent performance data available. Run some voting sessions first.'
                        }]
                    };
                }

                performanceText = `🤖 **All Agent Performance**\n\n` +
                    agents.map(([id, agent]) =>
                        `**${id}**\n` +
                        `   Wins: ${agent.wins}/${agent.totalResponses} (${(agent.winRate * 100).toFixed(1)}%)\n` +
                        `   Avg Confidence: ${agent.averageConfidence.toFixed(1)}/10\n` +
                        `   Trend: ${agent.trend}`
                    ).join('\n\n');
            }

            return {
                content: [{
                    type: 'text',
                    text: performanceText
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get agent performance: ${error.message}`
                }]
            };
        }
    }

    async handleAnalyzeTaskBreakdown(args) {
        const { topic, context = {} } = args;

        if (!topic || typeof topic !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid topic parameter. Please provide a string topic to analyze.'
                }]
            };
        }

        try {
            console.log(`🎯 Analyzing task breakdown for: "${topic}"`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'task_breakdown',
                { topic, context },
                async (params) => {
                    const breakdown = await this.taskManager.analyzeTaskBreakdown(params.topic, params.context);
                    const executionPlan = this.taskManager.getExecutionPlan(breakdown);
                    const optimalStrategy = await this.taskManager.selectOptimalStrategy(breakdown);

                    return { breakdown, executionPlan, optimalStrategy };
                }
            );

            const { breakdown, executionPlan, optimalStrategy } = result;

            // Collect analysis data
            await this.dataCollector.collectInteraction({
                type: 'task_breakdown',
                prompt: `Analyze task breakdown: ${topic}`,
                response: `Generated ${breakdown.subtasks.length} subtasks with ${executionPlan.phases.length} execution phases`,
                context: {
                    topic,
                    subtaskCount: breakdown.subtasks.length,
                    phaseCount: executionPlan.phases.length
                },
                success: true,
                sessionId: breakdown.id
            });

            return {
                content: [{
                    type: 'text',
                    text: `🎯 **Task Breakdown Analysis Complete**\n\n` +
                          `📝 **Main Objective:** ${breakdown.mainObjective}\n\n` +
                          `📋 **Subtasks Identified:** ${breakdown.subtasks.length}\n` +
                          breakdown.subtasks.map((task, i) =>
                              `${i + 1}. **${task.name}**\n   ${task.description}\n   Priority: ${task.priority} | Complexity: ${task.complexity}/10`
                          ).join('\n\n') + '\n\n' +
                          `🚀 **Execution Plan:** ${executionPlan.phases.length} phases\n` +
                          executionPlan.phases.map(phase =>
                              `Phase ${phase.phase}: ${phase.tasks.length} tasks${phase.canRunInParallel ? ' (parallel)' : ' (sequential)'}`
                          ).join('\n') + '\n\n' +
                          `💡 **Optimal Strategy:** ${optimalStrategy}\n\n` +
                          `🔗 **Breakdown ID:** ${breakdown.id}\n` +
                          `Use this ID with 'parallel_execute_tasks' to begin implementation.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Task breakdown analysis failed: ${error.message}\n\nPlease check your input and try again.`
                }]
            };
        }
    }

    async handleDeepThinkImplementation(args) {
        const { taskId, taskName, taskDescription, context = {} } = args;

        // Find task by ID or create temporary task
        let task;
        if (taskId) {
            task = this.taskManager.getTaskStatus(taskId);
            if (!task) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Task with ID "${taskId}" not found. Please use 'analyze_task_breakdown' first or provide taskName and taskDescription.`
                    }]
                };
            }
        } else if (taskName && taskDescription) {
            task = {
                id: `temp-${Date.now()}`,
                name: taskName,
                description: taskDescription
            };
        } else {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Please provide either taskId or both taskName and taskDescription.'
                }]
            };
        }

        try {
            console.log(`🧠 Starting deep thinking analysis for: "${task.name}"`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'deep_thinking',
                { taskId, taskName, taskDescription, context },
                async (params) => {
                    return await this.deepThinking.deepThinkImplementation(task, params.context);
                }
            );

            const thinkingSession = result;

            // Collect thinking data
            await this.dataCollector.collectInteraction({
                type: 'deep_thinking',
                prompt: `Deep think implementation: ${task.name}`,
                response: `Completed ${thinkingSession.phases.length} analysis phases`,
                context: {
                    taskId: task.id,
                    taskName: task.name,
                    phaseCount: thinkingSession.phases.length,
                    duration: thinkingSession.duration
                },
                success: true,
                sessionId: thinkingSession.id
            });

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **Deep Thinking Analysis Complete**\n\n` +
                          `📝 **Task:** ${task.name}\n` +
                          `⏱️ **Duration:** ${thinkingSession.duration}ms\n` +
                          `🔍 **Analysis Phases:** ${thinkingSession.phases.length}\n\n` +
                          `📊 **Analysis Summary:**\n` +
                          thinkingSession.phases.map(phase =>
                              `• ${phase.phase.replace('_', ' ').toUpperCase()}: Completed`
                          ).join('\n') + '\n\n' +
                          `💡 **Key Recommendations:**\n${thinkingSession.recommendations?.content || 'Analysis in progress'}\n\n` +
                          `🔗 **Session ID:** ${thinkingSession.id}\n` +
                          `Use this session data for implementation planning.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Deep thinking analysis failed: ${error.message}\n\nPlease check your input and try again.`
                }]
            };
        }
    }

    async handleParallelExecuteTasks(args) {
        const { breakdownId, executionStrategy = 'parallel', maxConcurrency = 5 } = args;

        if (!breakdownId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing breakdownId parameter. Please provide a breakdown ID from a previous task analysis.'
                }]
            };
        }

        // Find the breakdown
        const breakdown = this.taskManager.activeTasks.get(breakdownId);
        if (!breakdown) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Breakdown with ID "${breakdownId}" not found. Please run 'analyze_task_breakdown' first.`
                }]
            };
        }

        try {
            console.log(`⚡ Starting parallel execution for breakdown: ${breakdown.mainObjective}`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'parallel_execution',
                { breakdownId, executionStrategy, maxConcurrency },
                async (params) => {
                    const executionPlan = this.taskManager.getExecutionPlan(breakdown);
                    return await this.parallelImplementation.parallelExecuteTasks(breakdown, executionPlan);
                }
            );

            const execution = result;

            // Collect execution data
            await this.dataCollector.collectInteraction({
                type: 'parallel_execution',
                prompt: `Execute tasks in parallel: ${breakdown.mainObjective}`,
                response: `Executed ${breakdown.subtasks.length} tasks across ${execution.phases.length} phases`,
                context: {
                    breakdownId,
                    executionStrategy,
                    maxConcurrency,
                    taskCount: breakdown.subtasks.length,
                    phaseCount: execution.phases.length,
                    duration: execution.duration
                },
                success: execution.status === 'completed',
                sessionId: execution.id
            });

            const completedTasks = execution.completedTasks.size;
            const failedTasks = execution.failedTasks.size;
            const discoveredTasks = execution.discoveredTasks.length;

            return {
                content: [{
                    type: 'text',
                    text: `⚡ **Parallel Execution ${execution.status === 'completed' ? 'Complete' : 'Failed'}**\n\n` +
                          `📋 **Project:** ${breakdown.mainObjective}\n` +
                          `⏱️ **Duration:** ${execution.duration}ms\n` +
                          `📊 **Execution Summary:**\n` +
                          `   • Completed Tasks: ${completedTasks}/${breakdown.subtasks.length}\n` +
                          `   • Failed Tasks: ${failedTasks}\n` +
                          `   • Phases Executed: ${execution.phases.length}\n` +
                          `   • Discovered Tasks: ${discoveredTasks}\n\n` +
                          `🔄 **Phase Results:**\n` +
                          execution.phases.map(phase =>
                              `Phase ${phase.phase}: ${phase.tasks.filter(t => t.status === 'completed').length}/${phase.tasks.length} tasks completed`
                          ).join('\n') + '\n\n' +
                          (discoveredTasks > 0 ?
                              `💡 **Discovered Additional Tasks:**\n` +
                              execution.discoveredTasks.slice(0, 5).map(task =>
                                  `• ${task.name}: ${task.description}`
                              ).join('\n') +
                              (discoveredTasks > 5 ? `\n... and ${discoveredTasks - 5} more` : '') + '\n\n' : '') +
                          `🔗 **Execution ID:** ${execution.id}\n` +
                          `Status: ${execution.status}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Parallel execution failed: ${error.message}\n\nPlease check the breakdown ID and try again.`
                }]
            };
        }
    }

    setupErrorHandling() {
        this.server.onerror = (error) => {
            console.error('❌ MCP Server error:', error);
            // Don't exit on MCP errors, just log them
        };

        process.on('SIGINT', async () => {
            console.error('🛑 Shutting down ZAI MCP Server...');

            // Stop all active loops
            for (const loop of this.activeLoops.values()) {
                loop.status = 'stopped';
            }
            this.activeLoops.clear();

            try {
                await this.dataCollector.flushData();
                await this.server.close();
            } catch (error) {
                console.error('❌ Error during shutdown:', error);
            }
            process.exit(0);
        });

        process.on('uncaughtException', (error) => {
            console.error('❌ Uncaught exception:', error);
            // Only exit if it's not a buffer-related error
            if (!error.message.includes('subarray') && !error.message.includes('buffer')) {
                process.exit(1);
            }
        });

        process.on('unhandledRejection', (reason, promise) => {
            console.error('❌ Unhandled rejection at:', promise, 'reason:', reason);
            // Only exit if it's not a buffer-related error
            if (reason && !reason.toString().includes('subarray') && !reason.toString().includes('buffer')) {
                process.exit(1);
            }
        });
    }

    async start() {
        try {
            console.error('🚀 Starting ZAI MCP Server...');
            console.error('🔧 Initializing transport layer...');

            // Don't modify stdin before creating transport
            const transport = new StdioServerTransport();
            console.error('📡 Connecting to MCP transport...');

            await this.server.connect(transport);

            console.error('✅ ZAI MCP Server started successfully');
            console.error('📡 Listening for MCP messages...');
            console.error('🎯 Server ready for tool calls');

        } catch (error) {
            console.error('❌ Failed to start MCP server:', error);
            console.error('Error details:', error.stack);
            process.exit(1);
        }
    }
}

// Start the server
async function main() {
    try {
        console.error('🔧 Initializing ZAI MCP Server...');
        const server = new ZAIMCPServer();
        await server.start();
    } catch (error) {
        console.error('❌ Failed to start ZAI MCP Server:', error);
        console.error('Stack trace:', error.stack);
        process.exit(1);
    }
}

// Run if this is the main module
const isMainModule = import.meta.url === `file://${process.argv[1]}` ||
    process.argv[1]?.endsWith('src/index.js') ||
    process.argv[1]?.endsWith('zai-mcp-server') ||
    process.argv[1]?.includes('zai-mcp-server') ||
    process.argv.some(arg => arg.includes('zai-mcp-server'));

if (isMainModule) {
    main().catch((error) => {
        console.error('❌ Fatal error:', error);
        process.exit(1);
    });
}

export { ZAIMCPServer };
